package introspect;

import PrologDB.DB;
import PrologDB.DBSchema;
import PrologDB.Table;

public class Helper {
    // YOU NEED TO CHANGE THE TARGET STRING BELOW TO ONE OF YOUR CLASS FILES!!!
    static String target = "/Java/otherClasses/graff/CycleWorkSpace.class";

    /** no arguments to run this main(...) -- illustrates how to use
     * findClassForPath and reading and printing MDELite databases
     * @param args  -- ignored provide null
     * @throws Exception -- only when main(...) bombs -- it shouldn't
     */
    public static void main(String... args) throws Exception {
        // Step 1: make sure that class "graff.Main" exists on the classpath,
        //         otherwise throw an exception
        Class.forName("graff.Main");

        // Step 2: this is where we winnow down target to a package.name exists
        Class c = findClassForPath(target);
        System.out.println(c.getName());
        
        // Step 3: illustrate the creation of a database
        readAndPrintDB();
    }

    /**
     * a path is a linux path "/a/b/c/d.class"
     * @param path -- linux path to a .class file
     * @return Class object of referenced file
     */
    public static Class findClassForPath(String path) {
        String dotPath = path.replace(".class", "").replace('\\', '.').replace('/', '.');
        Class c;
        while (true) {
            try {
                c = Class.forName(dotPath);
                break;
            } catch (ClassNotFoundException e) {
                // Remove xxx. that leads target as this is not part of a
                // java class name
                int i = dotPath.indexOf('.');
                if (i == -1) {
                    throw new RuntimeException("there is no such target " + target);
                }
                dotPath = dotPath.substring(i + 1);
                continue;
            }
        }
        return c;
    }
    
    /**
     * illustrate how to read in a schema, create a database instance of
     * a schema, gets empty tables of this database and then populates
     * them with artificial tuples, and finally prints the database
     * to standard out
     */
    static void readAndPrintDB() {
        // Step 1: read the target schema
        DBSchema dbs = DBSchema.readDBSchema("prog1.schema.pl");
        
        // Step 2: instantiate the schema, the instance of which is called "intro"
        DB db = new DB("intro", dbs);
        
        // Step 3: retrieve the empty bcClass and bMember tables
        Table bcClass = db.getTable("bcClass");
        Table bcMember = db.getTable("bcMember");
        
        // Step 4: populate with artificial data -- you will have to supply
        //         real data for these tables

        for (int i = 0 ; i < 4 ; i++) {
            String cid = "c"+i;
            String cname = "Class"+i;
            String csuper= "Object";
            bcClass.addTuple(cid, cname, csuper);
            
            for (int j=0; j<2; j++) {
                String mid = "m"+i+j;
                String statik = (i%2==0)?"true":"false";
                String type = "int";
                String sig = "foo"+j+"(int)";
                bcMember.addTuple(mid,cid, statik, type, sig);
            }
        }
        
        // Step 5: print the artificial database to System.out
        db.print(System.out);   
    }
}
